// Reso: http://www.bcheck.net/apps/
// apps@bcheck.net

// 2003-05-26: vastly simplified and shrunk
// 2006-03-29: refresh rate selection, cleanup
// 2006-11-04: pass additional arguments
// 2006-11-05: own dialog rather than hooked file dialog

#include <windows.h>
#include <shlobj.h>
#include "resource.h"
#include <string.h>

#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")

const char* g_szTitle = "Res-O-Matic";
HINSTANCE g_hInst;
HWND g_hwnd;
char g_szResoPath[MAX_PATH];

void RunResoApp(char* cmdline);

void Warn(const char* szErr) {
	MessageBox(g_hwnd, szErr, g_szTitle, MB_ICONEXCLAMATION);
}

void* mymalloc(size_t bytes) {
	void* mem;
	if(!bytes) return NULL;
	mem=HeapAlloc(GetProcessHeap(), 0, bytes);
	if(!mem)
		Warn("MemErr");
	return mem;
}

void myfree(void* mem) {
	if(mem)
		HeapFree(GetProcessHeap(), 0, mem);
}

char* _tok;
char* mystrtok(char* sz, const char* seps) {
	const char* p;

	if(sz) _tok=sz;
	else sz=_tok;

	while(*_tok) {
		for(p=seps; *p; p++)
			if(*_tok==*p) {
				*_tok=0;
				_tok++;
				return sz;
			}
		_tok++;
	}
	if(sz==_tok)
		return NULL;
	else
		return sz;
}

int myatoi(const char* a) {
	int i=0,neg=1;
	if(!a) return 0;
	while(*a==' ') a++;
	if(*a=='-') {
		a++;
		neg=-1;
	}
	while(*a && *a>='0' && *a<='9')
		i = i*10 + *(a++)-'0';
	return i*neg;
}

static void swap(void *x, void *y, size_t l) {
   char *a = x, *b = y, c;
   while(l--) {
      c = *a;
      *a++ = *b;
      *b++ = c;
   }
}

static void sort(char *array, size_t size, int (*cmp)(void*,void*), int begin, int end) {
   if (end > begin) {
      void *pivot = array + begin;
      int l = begin + size;
      int r = end;
      while(l < r) {
         if (cmp(array+l,pivot) <= 0) {
            l += size;
         } else {
            r -= size;
            swap(array+l, array+r, size); 
         }
      }
      l -= size;
      swap(array+begin, array+l, size);
      sort(array, size, cmp, begin, l);
      sort(array, size, cmp, r, end);
   }
}

void myqsort(void *array, size_t nitems, size_t size, int (*cmp)(void*,void*)) {
   sort(array, size, cmp, 0, (nitems-1)*size);
}

HRESULT CreateStartupLink(const char* szShortcut,
	const char* szRunFile, const char* szArguments, const char* szWorkingDir, const char* szIconFile)
{
	HRESULT hres;
	IShellLink* psl;
	
	hres=OleInitialize(NULL);
	if(hres == S_FALSE || hres == S_OK) {
		
		hres = CoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
			&IID_IShellLink, (void **) &psl);
		if(SUCCEEDED(hres)) {
			IPersistFile* ppf;
			
			hres = psl->lpVtbl->QueryInterface(psl, &IID_IPersistFile, (void **)&ppf);
			if(SUCCEEDED(hres)) {
				{
					hres = psl->lpVtbl->SetPath(psl, szRunFile);
					psl->lpVtbl->SetWorkingDirectory(psl, szWorkingDir);
					psl->lpVtbl->SetArguments(psl, szArguments);
					psl->lpVtbl->SetIconLocation(psl, szIconFile, 0);
				}
				if(SUCCEEDED(hres)) {
					WCHAR wsz[MAX_PATH];
					wsprintfW(wsz, L"%S", szShortcut);
					hres=ppf->lpVtbl->Save(ppf, (const WCHAR*)wsz, TRUE);
				}
				ppf->lpVtbl->Release(ppf);
			}
			psl->lpVtbl->Release(psl);
		}
		OleUninitialize();
	}
	return hres;
}

/* add item, but only if it's not already in the list */
void AddAndSelItem(HWND hwnd, const char* szText, LPARAM lpData, BOOL bSelect) {
	int iItem = SendMessage(hwnd, CB_FINDSTRINGEXACT, -1, (LPARAM)szText);
	if(iItem == CB_ERR) {
		iItem = SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM)szText);
		if(lpData!=0)
			SendMessage(hwnd, CB_SETITEMDATA, (WPARAM) iItem, (LPARAM) lpData);
		if(bSelect)
			SendMessage(hwnd, CB_SETCURSEL, (WPARAM)iItem, 0);
	}
}

int compareDevMode( const void *arg1, const void *arg2 )
{
	const DEVMODE *dma=arg1;
	const DEVMODE *dmb=arg2;

	// compare resolutions
	if(dma->dmPelsWidth > dmb->dmPelsWidth) return 1;
	if(dma->dmPelsWidth < dmb->dmPelsWidth) return -1;

	if(dma->dmPelsHeight > dmb->dmPelsHeight) return 1;
	if(dma->dmPelsHeight < dmb->dmPelsHeight) return -1;

	// compare colors
	if(dma->dmBitsPerPel > dmb->dmBitsPerPel) return 1;
	if(dma->dmBitsPerPel < dmb->dmBitsPerPel) return -1;

	// compare refresh rates
	if(dma->dmDisplayFrequency > dmb->dmDisplayFrequency) return 1;
	if(dma->dmDisplayFrequency < dmb->dmDisplayFrequency) return -1;

	return 0;
}

void ShowResolutions(HWND hdlg, DEVMODE* dmArray, int dmCount, int selWidth, int selHeight) {
	int iterDisplay;
	char szComboText[MAX_PATH];
	HWND hwndCombo;
	// add unique resolutions
	hwndCombo = GetDlgItem(hdlg, IDC_RESOLUTION);
	SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
	for(iterDisplay=0;iterDisplay<dmCount;iterDisplay++) {
		wsprintf(szComboText, "%dx%d", dmArray[iterDisplay].dmPelsWidth, dmArray[iterDisplay].dmPelsHeight);
		AddAndSelItem(hwndCombo, szComboText, (LPARAM)&dmArray[iterDisplay], 
			(dmArray[iterDisplay].dmPelsWidth==selWidth && dmArray[iterDisplay].dmPelsHeight==selHeight)
			);
	}
}

void ShowColors(HWND hdlg, DEVMODE* dmArray, int dmCount, int selWidth, int selHeight, int selColors) {
	int iterDisplay;
	char szComboText[MAX_PATH];
	// add unique color depths
	HWND hwndCombo = GetDlgItem(hdlg, IDC_COLORS);
	SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
	for(iterDisplay=0;iterDisplay<dmCount;iterDisplay++)
	{
		if(selWidth==dmArray[iterDisplay].dmPelsWidth && selHeight==dmArray[iterDisplay].dmPelsHeight)
		{
			wsprintf(szComboText, "%d bit color", dmArray[iterDisplay].dmBitsPerPel);
			AddAndSelItem(hwndCombo, szComboText, (LPARAM)dmArray[iterDisplay].dmBitsPerPel,
				(dmArray[iterDisplay].dmBitsPerPel==selColors)
				);
		}
	}
}

void ShowRefreshRates(HWND hdlg, DEVMODE* dmArray, int dmCount, int selWidth, int selHeight, int selRefreshRate) {
	int iterDisplay;
	char szComboText[MAX_PATH];
	// add unique refresh rates
	HWND hwndCombo = GetDlgItem(hdlg, IDC_REFRESH_RATE);
	SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
	for(iterDisplay=0;iterDisplay<dmCount;iterDisplay++)
	{
		if(selWidth==dmArray[iterDisplay].dmPelsWidth && selHeight==dmArray[iterDisplay].dmPelsHeight)
		{
			wsprintf(szComboText, "%d Hertz", dmArray[iterDisplay].dmDisplayFrequency);
			AddAndSelItem(hwndCombo, szComboText, (LPARAM)dmArray[iterDisplay].dmDisplayFrequency,
				(dmArray[iterDisplay].dmDisplayFrequency==selRefreshRate)
				);
		}
	}
}

DEVMODE *g_dmArray;
int g_dmCount;

void FillColorsAndRefreshRatesFromResolutions(HWND hdlg) {
	int curSel, curWidth, curHeight, curColors, curRefreshRate;
	DEVMODE* dm;
	HDC hdc = GetDC(NULL);
	HWND hwndResolution = GetDlgItem(hdlg, IDC_RESOLUTION);
	curSel = SendMessage(hwndResolution, CB_GETCURSEL, 0, 0);
	dm = (DEVMODE*)SendMessage(hwndResolution, CB_GETITEMDATA, curSel, 0);
	curWidth = dm->dmPelsWidth;
	curHeight = dm->dmPelsHeight;
	curColors = GetDeviceCaps(hdc, BITSPIXEL);
	curRefreshRate = GetDeviceCaps(hdc, VREFRESH);

	curSel=SendDlgItemMessage(hdlg, IDC_COLORS, CB_GETCURSEL, 0, 0);
	if(curSel!=-1)
		curColors = SendDlgItemMessage(hdlg, IDC_COLORS, CB_GETITEMDATA, curSel, 0);
	ShowColors(hdlg, g_dmArray, g_dmCount, curWidth, curHeight, curColors);

	curSel=SendDlgItemMessage(hdlg, IDC_REFRESH_RATE, CB_GETCURSEL, 0, 0);
	if(curSel!=-1)
		curRefreshRate = SendDlgItemMessage(hdlg, IDC_REFRESH_RATE, CB_GETITEMDATA, curSel, 0);
	ShowRefreshRates(hdlg, g_dmArray, g_dmCount, curWidth, curHeight, curRefreshRate);
}

void InitDialog(HWND hdlg) {
	DEVMODE dmTemp;
	int iterDisplay;
	BOOL success=TRUE;

	int curWidth, curHeight;
	HDC hdc = GetDC(NULL);

	// get current settings
	curWidth = GetDeviceCaps(hdc, HORZRES);
	curHeight = GetDeviceCaps(hdc, VERTRES);

	// count number of display settings
	dmTemp.dmSize = sizeof(DEVMODE);
	dmTemp.dmDriverExtra = 0;
	for(g_dmCount=0;EnumDisplaySettings(NULL, g_dmCount, &dmTemp);g_dmCount++);

	// get DEVMODEs into array
	g_dmArray = mymalloc(sizeof(DEVMODE)*g_dmCount);
	for(iterDisplay=0;success && iterDisplay<g_dmCount;iterDisplay++) {
		g_dmArray[iterDisplay].dmSize = sizeof(DEVMODE);
		g_dmArray[iterDisplay].dmDriverExtra = 0;
		success = EnumDisplaySettings(NULL, iterDisplay, &g_dmArray[iterDisplay]);
	}

	// sort the array
	myqsort((void *)g_dmArray, (size_t)g_dmCount+1, sizeof(DEVMODE), compareDevMode); // this is +1 for no good reason

	// fill the combo boxen
	ShowResolutions(hdlg, g_dmArray, g_dmCount, curWidth, curHeight);
	FillColorsAndRefreshRatesFromResolutions(hdlg);

	// global hwnd
	g_hwnd = hdlg;
	SetClassLong(hdlg,GCL_HICON,(long)LoadIcon(g_hInst,MAKEINTRESOURCE(IDI_RESO)) );
	{
		HMENU hMenu = GetSystemMenu(hdlg, FALSE);
		AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
		AppendMenu(hMenu, MF_STRING, 123,  "About Res-O-Matic...");
	}
}

// given szFile = "c:\pathname\stuff\application.exe"
// return NameStart = "application.exe"
//        ExtStart  = ".exe"
void CrackFilename(char* szFile, char** ppNameStart, char** ppExtStart) {
	char* NameStart, *ExtStart;
	NameStart = strrchr(szFile, '\\');
	if(!NameStart) NameStart = szFile;
	else NameStart++;
	if(ppExtStart) {
		ExtStart = strrchr(NameStart, '.');
		if(!ExtStart) ExtStart = NameStart+lstrlen(NameStart);
		*ppExtStart = ExtStart;
	}
	if(ppNameStart)
		*ppNameStart = NameStart;
}

BOOL GetAppToRun(char *szFile, DWORD ccbFile) {
	OPENFILENAME ofn;
	szFile[0]=0;

	/* Initialize OPENFILENAME struct */
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = "Programs (*.exe,*.com,*.bat)\0*.exe;*.com;*.bat\0All Files (*.*)\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrInitialDir = NULL; //"c:\\";
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = ccbFile;
	ofn.lpstrTitle = "Select Application";
	ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_ENABLESIZING;
	ofn.lpstrDefExt = ofn.lpstrCustomFilter = ofn.lpstrFileTitle = NULL;
	ofn.hwndOwner = g_hwnd;
	ofn.hInstance = g_hInst;

	/* Display the Open dialog box */
	return GetOpenFileName(&ofn);
}

void queryShellFolders(char *name, char *out) {
	HKEY hKey;
	if ( RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",0,KEY_READ,&hKey) == ERROR_SUCCESS) {
		int l = MAX_PATH;
		int t = REG_SZ;
		RegQueryValueEx(hKey,name,NULL,&t,out,&l );
		RegCloseKey(hKey);
	}
}

/* ask user for location to save shortcut */
BOOL GetSaveLocation(char* szFile, int cbszFile) {
	OPENFILENAME ofn;

	/* Initialize OPENFILENAME struct */
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = "Shortcut File (*.lnk)\0*.lnk\0All Files (*.*)\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrInitialDir = NULL; //"c:\\";
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = cbszFile;
	ofn.lpstrTitle = "Create Shortcut";
	ofn.Flags = OFN_HIDEREADONLY | OFN_EXPLORER | OFN_ENABLESIZING;
	ofn.lpstrDefExt = ofn.lpstrCustomFilter = ofn.lpstrFileTitle = NULL;
	ofn.hwndOwner = g_hwnd;
	ofn.hInstance = g_hInst;

	/* Display the Save dialog box */
	return GetSaveFileName(&ofn);
}

void GetUserOptionsAsString(HWND hdlg, char* szArguments, char* runApplication, int *width, int *height)
{
	int colors, refreshRate;

	HWND hwndResolution = GetDlgItem(hdlg, IDC_RESOLUTION);
	int curSel = SendMessage(hwndResolution, CB_GETCURSEL, 0, 0);
	DEVMODE* dm = (DEVMODE*)SendMessage(hwndResolution, CB_GETITEMDATA, curSel, 0);
	*width = dm->dmPelsWidth;
	*height = dm->dmPelsHeight;

	curSel=SendDlgItemMessage(hdlg, IDC_COLORS, CB_GETCURSEL, 0, 0);
	if(curSel!=-1)
		colors = SendDlgItemMessage(hdlg, IDC_COLORS, CB_GETITEMDATA, curSel, 0);

	curSel=SendDlgItemMessage(hdlg, IDC_REFRESH_RATE, CB_GETCURSEL, 0, 0);
	if(curSel!=-1)
		refreshRate = SendDlgItemMessage(hdlg, IDC_REFRESH_RATE, CB_GETITEMDATA, curSel, 0);

	GetDlgItemText(hdlg, IDC_APPLICATION, runApplication, MAX_PATH);

	wsprintf(szArguments, "\"%s\" %d %d %d %d", runApplication, *width, *height, colors, refreshRate);
}

UINT APIENTRY DialogProc(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uiMsg)
	{
	case WM_INITDIALOG:
		InitDialog(hdlg);
		return FALSE;
	case WM_CLOSE:
		EndDialog(hdlg,0);
		return TRUE;
	case WM_DESTROY:
		myfree(g_dmArray);
		return TRUE;
	case WM_SYSCOMMAND:
		if(wParam == 123) {
			MessageBox(hdlg, "Res-O-Matic (build 5)\r\nby Tim Rice\r\n\r\nAngelDust Research ltd.",
				"About Res-O-Matic...", MB_ICONINFORMATION | MB_RIGHT);
			return TRUE;
		}
		return FALSE;
	case WM_COMMAND:
		if(wParam==IDCANCEL) {
			EndDialog(hdlg,0);
			return TRUE;
		} else if(wParam==IDC_BROWSE) {
			char szFile[MAX_PATH];
			if(GetAppToRun(szFile, sizeof(szFile)))
			{
				SetDlgItemText(hdlg, IDC_APPLICATION, szFile);
			}
			return TRUE;
#ifdef TESTBUTTON
		} else if(wParam==IDC_TEST) {
			char szArguments[MAX_PATH], runApplication[MAX_PATH];
			int width, height;
			GetUserOptionsAsString(hdlg, szArguments, runApplication, &width, &height);
			/* make sure application exists */
			{
				DWORD attr = GetFileAttributes(runApplication);
				if(attr==INVALID_FILE_ATTRIBUTES || attr&FILE_ATTRIBUTE_DIRECTORY || attr&FILE_ATTRIBUTE_DEVICE) {
					Warn("Please select a valid application first.");
					return TRUE;
				}
			}
			RunResoApp(szArguments);
			return TRUE;
#endif
		} else if(wParam==IDC_CREATESHORTCUT) {
			/* get user options */
			char szArguments[MAX_PATH], runApplication[MAX_PATH];
			int width, height;
			GetUserOptionsAsString(hdlg, szArguments, runApplication, &width, &height);

			/* make sure application exists */
			{
				DWORD attr = GetFileAttributes(runApplication);
				if(attr==INVALID_FILE_ATTRIBUTES || attr&FILE_ATTRIBUTE_DIRECTORY || attr&FILE_ATTRIBUTE_DEVICE) {
					Warn("Please select a valid application first.");
					return TRUE;
				}
			}

			/* create shortcut */
			{
				char szShortcut[MAX_PATH];
				char *nameStart, *extStart;

				/* create default shortcut name */
				CrackFilename(runApplication, &nameStart, &extStart);
				queryShellFolders("Programs", szShortcut);
				lstrcat(szShortcut, nameStart-1);
				wsprintf(&szShortcut[lstrlen(szShortcut)-lstrlen(extStart)], " (at %dx%d).lnk", width, height);

				/* ask user where to save shortcut */
				if(GetSaveLocation(szShortcut, sizeof(szShortcut))==TRUE) {
					BOOL bSuccess;

					/* setup reso arguments for application to run and display mode */
					char szWorkingDir[MAX_PATH];
					lstrcpy(szWorkingDir, runApplication);
					szWorkingDir[nameStart-runApplication]=0;

					/* create the shortcut */
					bSuccess = SUCCEEDED(CreateStartupLink(szShortcut, g_szResoPath, szArguments, szWorkingDir, runApplication));
					MessageBox(g_hwnd, (bSuccess ? "Shortcut created." : "Unable to create shortcut"),
						g_szTitle, (bSuccess ? MB_ICONINFORMATION : MB_ICONEXCLAMATION) );
				}
			}		
			return TRUE;
		} else if(HIWORD(wParam)==CBN_SELENDOK && (HWND)lParam==GetDlgItem(hdlg, IDC_RESOLUTION)) {
			FillColorsAndRefreshRatesFromResolutions(hdlg);
			return TRUE;
		}
	}
	return FALSE;
}

void RunAndWait(const char* szProgram, const char* szArguments, const char* szDirectory, int nCmdShow)
{
	SHELLEXECUTEINFO sei;
	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.fMask = SEE_MASK_NOCLOSEPROCESS;
	sei.lpFile = szProgram;
	sei.lpParameters = szArguments;
	sei.nShow = nCmdShow;
	sei.hwnd = NULL;
	sei.lpVerb = NULL;
	sei.lpDirectory = szDirectory;
	// ShellExecuteEx outputs to hInstApp and hProcess members of sei structure

	// start the program "szProgram"
	if(!ShellExecuteEx(&sei))
		Warn("Res-o-matic Error:\n\nError starting application");
	else if(WaitForSingleObject(sei.hProcess, INFINITE) == WAIT_FAILED) // wait for program to exit
		Warn("Res-o-matic Error:\n\nError waiting for application to exit");

}

const char* GetDispChangeErrorMessage(LONG error) {
	switch(error) {
		case DISP_CHANGE_BADMODE: return "Graphics mode is not supported";
		case DISP_CHANGE_FAILED: return "Display driver failed specified graphics mode";
		case DISP_CHANGE_RESTART: return "Must restart for graphics mode";
		default: return "Unable to change resolution";
	}
}

/* truncate and return next quotable string in cmdline, note cmdline is changed */
char* GetNextQuotedString(char* cmdline, char** quotedString) {
	if (*cmdline == '\"') {
		cmdline++;
		*quotedString = cmdline; /* save for setup */
		while (*cmdline && *cmdline != '\"') cmdline++;
		if (*cmdline) {
			*cmdline=0; /* replace closing quote with trunc */
			cmdline++;
		}
	} else {
		*quotedString = cmdline; /* save for setup */
		while (*cmdline && *cmdline != ' ') cmdline++;
		if (*cmdline) {
			*cmdline=0; /* replace space with trunc */
			cmdline++;
		}
	}
	while (*cmdline == ' ') cmdline++;
	return cmdline;
}

/* cmdline here is actually:
"APPTORUN.EXE" 1024 768 32 60
*/
void RunResoApp(char* cmdline) {
	HDC hdc;
	int width, height, colors, refreshRate;
	char *nextArgument; /* argument iterator */
	char *extraArguments; /* extra command line arguments, pass these on */
	char* szRunApp;

	/* get current resolution as default */
	hdc = GetDC(NULL);
	width = GetDeviceCaps(hdc, HORZRES);
	height = GetDeviceCaps(hdc, VERTRES);
	colors = GetDeviceCaps(hdc, BITSPIXEL);
	refreshRate = GetDeviceCaps(hdc, VREFRESH);

	/* get program to run */
	cmdline = GetNextQuotedString(cmdline, &szRunApp);

	/* get settings off command line */
	nextArgument = mystrtok(cmdline, " ");
	if(nextArgument) {
		width = myatoi(nextArgument);
		nextArgument = mystrtok(NULL, " ");
		if(nextArgument) {
			height = myatoi(nextArgument);
			nextArgument = mystrtok(NULL, " ");
			if(nextArgument) {
				colors = myatoi(nextArgument);
				nextArgument = mystrtok(NULL, " ");
				if(nextArgument) {
					refreshRate = myatoi(nextArgument);
					extraArguments = _tok;
				}
			}
		}
	}

	{
		/* find resolution of choice */
		LONG result;
		DEVMODE dm;
		dm.dmSize = sizeof(DEVMODE);
		dm.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
		dm.dmPelsWidth = width;
		dm.dmPelsHeight = height;
		dm.dmBitsPerPel = colors;
		dm.dmDisplayFrequency = refreshRate;

		/* change resolution */
		result = ChangeDisplaySettings(&dm, 0);
		if(result!=DISP_CHANGE_SUCCESSFUL) {
			char duh[MAX_PATH];
			wsprintf(duh, "Unable to change resolution to %dx%d at %d bpp (%d Hz)\n\n%s",
				width,height,colors,refreshRate, GetDispChangeErrorMessage(result));
			Warn(duh);
		} else {
			char szDirectory[MAX_PATH];
			GetCurrentDirectory(sizeof(szDirectory), szDirectory);
			/* run program and wait for it to exit */
			RunAndWait(szRunApp, extraArguments, szDirectory, SW_SHOWNORMAL);

			/* change back to default resolution */
			result = ChangeDisplaySettings(NULL, 0);
		}
	}
}

/* note WinMain arguments are invalid if no CRT */
int WINAPI WinMain(HINSTANCE hInstanceNOUSE, HINSTANCE hPrevInstanceNOUSE,
				   LPSTR cmdlineNOUSE, int nCmdShowNOUSE) {
	char *cmdline, *szResoPath;
	g_hInst = GetModuleHandle(NULL);
	cmdline = GetCommandLine();

	/* get path to current running Reso (for making shortcuts) */
	cmdline = GetNextQuotedString(cmdline, &szResoPath);
	lstrcpy(g_szResoPath, szResoPath);

	/* run setup or run app */	
	if(*cmdline==0) {
		/* if there are no arguments, run Reso's setup */
		DialogBox(g_hInst, MAKEINTRESOURCE(IDD_DLG_RESO), NULL, DialogProc);
	} else {
		/* otherwise change res, run program, change res back */
		RunResoApp(cmdline);
	}

	ExitProcess(0); /* must explicitly call when no CRT */
}